import 'package:flutter/material.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/space_details/presentation/helpers/space_details_dialog_helper.dart';
import 'package:syncrow_web/utils/color_manager.dart';

class CreateSpaceButton extends StatefulWidget {
  const CreateSpaceButton({
    required this.communityUuid,
    super.key,
  });

  final String communityUuid;

  @override
  State<CreateSpaceButton> createState() => _CreateSpaceButtonState();
}

class _CreateSpaceButtonState extends State<CreateSpaceButton> {
  bool _isHovered = false;

  @override
  Widget build(BuildContext context) {
    return Tooltip(
      margin: const EdgeInsets.symmetric(vertical: 24),
      message: 'Create a new space',
      child: GestureDetector(
        onTap: () => SpaceDetailsDialogHelper.showCreate(
          context,
          communityUuid: widget.communityUuid,
        ),
        child: MouseRegion(
          onEnter: (_) => setState(() => _isHovered = true),
          onExit: (_) => setState(() => _isHovered = false),
          child: AnimatedOpacity(
            duration: const Duration(milliseconds: 100),
            opacity: _isHovered ? 1.0 : 0.45,
            child: Container(
              width: 150,
              height: 90,
              decoration: BoxDecoration(
                color: Colors.white,
                borderRadius: BorderRadius.circular(20),
                boxShadow: [
                  BoxShadow(
                    color: Colors.grey.withValues(alpha: 0.2),
                    spreadRadius: 3,
                    blurRadius: 8,
                    offset: const Offset(0, 4),
                  ),
                ],
              ),
              child: Container(
                margin: const EdgeInsets.symmetric(vertical: 20),
                decoration: BoxDecoration(
                  border: Border.all(color: ColorsManager.borderColor, width: 2),
                  color: ColorsManager.boxColor,
                  shape: BoxShape.circle,
                ),
                child: const Center(
                  child: Icon(
                    Icons.add,
                    color: Colors.blue,
                  ),
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }
}
