import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/communities/domain/models/space_model.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/space_details/data/services/remote_space_details_service.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/space_details/domain/models/space_details_model.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/space_details/presentation/bloc/space_details_bloc.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/space_details/presentation/widgets/space_details_dialog.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/update_space/data/services/remote_update_space_service.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/update_space/domain/params/update_space_param.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/update_space/presentation/bloc/update_space_bloc.dart';
import 'package:syncrow_web/services/api/http_service.dart';

abstract final class SpaceDetailsDialogHelper {
  static void showCreate(
    BuildContext context, {
    required String communityUuid,
  }) {
    showDialog<void>(
      context: context,
      builder: (_) => MultiBlocProvider(
        providers: [
          BlocProvider(
            create: (context) => SpaceDetailsBloc(
              RemoteSpaceDetailsService(httpService: HTTPService()),
            ),
          ),
          BlocProvider(
            create: (context) => UpdateSpaceBloc(
              RemoteUpdateSpaceService(HTTPService()),
            ),
          ),
        ],
        child: Builder(
          builder: (context) => SpaceDetailsDialog(
            context: context,
            title: const SelectableText('Create Space'),
            spaceModel: SpaceModel.empty(),
            onSave: (space) {},
            communityUuid: communityUuid,
          ),
        ),
      ),
    );
  }

  static void showEdit(
    BuildContext context, {
    required SpaceModel spaceModel,
    required String communityUuid,
  }) {
    showDialog<void>(
      context: context,
      builder: (_) => MultiBlocProvider(
        providers: [
          BlocProvider(
            create: (context) => SpaceDetailsBloc(
              RemoteSpaceDetailsService(httpService: HTTPService()),
            ),
          ),
          BlocProvider(
            create: (context) => UpdateSpaceBloc(
              RemoteUpdateSpaceService(HTTPService()),
            ),
          ),
        ],
        child: Builder(
          builder: (context) => BlocListener<UpdateSpaceBloc, UpdateSpaceState>(
            listener: _updateListener,
            child: SpaceDetailsDialog(
              context: context,
              title: const SelectableText('Edit Space'),
              spaceModel: spaceModel,
              onSave: (space) => context.read<UpdateSpaceBloc>().add(
                    UpdateSpace(
                      UpdateSpaceParam(
                        communityUuid: communityUuid,
                        space: space,
                      ),
                    ),
                  ),
              communityUuid: communityUuid,
            ),
          ),
        ),
      ),
    );
  }

  static void _updateListener(BuildContext context, UpdateSpaceState state) {
    return switch (state) {
      UpdateSpaceInitial() => null,
      UpdateSpaceLoading() => _onLoading(context),
      UpdateSpaceSuccess(:final space) => _onUpdateSuccess(context, space),
      UpdateSpaceFailure(:final errorMessage) => _onError(context, errorMessage),
    };
  }

  static void _onUpdateSuccess(BuildContext context, SpaceDetailsModel space) {
    Navigator.of(context).pop();
  }

  static void _onLoading(BuildContext context) {
    showDialog<void>(
      context: context,
      barrierDismissible: false,
      builder: (_) => const Center(child: CircularProgressIndicator()),
    );
  }

  static void _onError(BuildContext context, String errorMessage) {
    Navigator.of(context).pop();
    showDialog<void>(
      context: context,
      barrierDismissible: false,
      builder: (_) => AlertDialog(
        title: const Text('Error'),
        content: Text(errorMessage),
        actions: [
          TextButton(
            onPressed: Navigator.of(context).pop,
            child: const Text('OK'),
          ),
        ],
      ),
    );
  }
}
