import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/products/data/services/remote_products_service.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/products/domain/models/product.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/products/presentation/bloc/products_bloc.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/space_details/presentation/widgets/space_details_action_buttons.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/tags/presentation/widgets/products_grid.dart';
import 'package:syncrow_web/services/api/http_service.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/extension/build_context_x.dart';

class AddDeviceTypeWidget extends StatefulWidget {
  const AddDeviceTypeWidget({super.key});

  @override
  State<AddDeviceTypeWidget> createState() => _AddDeviceTypeWidgetState();
}

class _AddDeviceTypeWidgetState extends State<AddDeviceTypeWidget> {
  final Map<Product, int> _selectedProducts = {};

  void _onIncrement(Product product) {
    setState(() {
      _selectedProducts[product] = (_selectedProducts[product] ?? 0) + 1;
    });
  }

  void _onDecrement(Product product) {
    setState(() {
      if ((_selectedProducts[product] ?? 0) > 0) {
        _selectedProducts[product] = _selectedProducts[product]! - 1;
        if (_selectedProducts[product] == 0) {
          _selectedProducts.remove(product);
        }
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return BlocProvider(
      create: (_) => ProductsBloc(RemoteProductsService(HTTPService()))
        ..add(const LoadProducts()),
      child: Builder(
        builder: (context) => AlertDialog(
          title: const SelectableText('Add Devices'),
          backgroundColor: ColorsManager.whiteColors,
          content: BlocBuilder<ProductsBloc, ProductsState>(
            builder: (context, state) => switch (state) {
              ProductsInitial() || ProductsLoading() => _buildLoading(context),
              ProductsLoaded(:final products) => ProductsGrid(
                  products: products,
                  selectedProducts: _selectedProducts,
                  onIncrement: _onIncrement,
                  onDecrement: _onDecrement,
                ),
              ProductsFailure(:final errorMessage) => _buildFailure(
                  context,
                  errorMessage,
                ),
            },
          ),
          actions: [
            SpaceDetailsActionButtons(
              onSave: () {
                final result = _selectedProducts.entries
                    .expand((entry) => List.generate(entry.value, (_) => entry.key))
                    .toList();
                Navigator.of(context).pop(result);
              },
              onCancel: Navigator.of(context).pop,
              saveButtonLabel: 'Next',
            ),
          ],
        ),
      ),
    );
  }

  Widget _buildLoading(BuildContext context) => SizedBox(
        width: context.screenWidth * 0.9,
        height: context.screenHeight * 0.65,
        child: const Center(child: CircularProgressIndicator()),
      );

  Widget _buildFailure(BuildContext context, String errorMessage) {
    return SizedBox(
      width: context.screenWidth * 0.9,
      height: context.screenHeight * 0.65,
      child: Center(
        child: SelectableText(
          errorMessage,
          style: context.textTheme.bodyMedium?.copyWith(
            color: context.theme.colorScheme.error,
          ),
        ),
      ),
    );
  }
}
