import 'package:dio/dio.dart';
import 'package:syncrow_web/pages/common/bloc/project_manager.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/space_details/domain/models/space_details_model.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/update_space/domain/params/update_space_param.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/update_space/domain/services/update_space_service.dart';
import 'package:syncrow_web/services/api/api_exception.dart';
import 'package:syncrow_web/services/api/http_service.dart';

class RemoteUpdateSpaceService implements UpdateSpaceService {
  const RemoteUpdateSpaceService(this._httpService);

  final HTTPService _httpService;

  static const _defaultErrorMessage = 'Failed to update space';

  @override
  Future<SpaceDetailsModel> updateSpace(UpdateSpaceParam param) async {
    try {
      final path = await _makeUrl(param);
      await _httpService.put(
        path: path,
        body: param.space.toJson(),
        expectedResponseModel: (data) {
          final response = data as Map<String, dynamic>;
          final isSuccess = response['success'] as bool;
          if (!isSuccess) {
            throw APIException(response['error'] as String);
          }
          return isSuccess;
        },
      );

      return param.space;
    } on DioException catch (e) {
      final message = e.response?.data as Map<String, dynamic>?;
      final error = message?['error'] as Map<String, dynamic>?;
      final errorMessage = error?['error'] as String? ?? '';
      final formattedErrorMessage = [
        _defaultErrorMessage,
        errorMessage,
      ].join(': ');
      throw APIException(formattedErrorMessage);
    } catch (e) {
      final formattedErrorMessage = [_defaultErrorMessage, '$e'].join(': ');
      throw APIException(formattedErrorMessage);
    }
  }

  Future<String> _makeUrl(UpdateSpaceParam param) async {
    final projectUuid = await ProjectManager.getProjectUUID();
    if (projectUuid == null || projectUuid.isEmpty) {
      throw APIException('Project UUID is not set');
    }

    final spaceUuid = param.space.uuid;
    if (spaceUuid.isEmpty) {
      throw APIException('Space UUID is not set');
    }

    final communityUuid = param.communityUuid;
    if (communityUuid.isEmpty) {
      throw APIException('Community UUID is not set');
    }

    return '/projects/$projectUuid/communities/$communityUuid/spaces/$spaceUuid';
  }
}
